@@ -5,12 +5,13 @@ use std::fmt::Display;
 use std::path::PathBuf;
 use std::{collections::BTreeMap, env};
 
+use crate::repository::IndexedIdsRepo;
 use crate::{
     Application, RUSTIC_APP,
     commands::{init::init, snapshots::fill_table},
     config::{hooks::Hooks, parse_labels},
     helpers::{bold_cell, bytes_size_to_string, table},
-    repository::CliRepo,
+    repository::Repo,
     status_err,
 };
 
@@ -20,14 +21,13 @@ use clap::ValueHint;
 use comfy_table::Cell;
 use conflate::{Merge, MergeFrom};
 use log::{debug, error, info, warn};
-use rustic_core::StringList;
+use rustic_core::{Excludes, StringList};
 use serde::{Deserialize, Serialize};
 use serde_with::serde_as;
 
 use rustic_core::{
-    BackupOptions, CommandInput, ConfigOptions, IndexedIds, KeyOptions, LocalSourceFilterOptions,
-    LocalSourceSaveOptions, ParentOptions, PathList, ProgressBars, Repository, SnapshotOptions,
-    repofile::SnapshotFile,
+    BackupOptions, CommandInput, ConfigOptions, KeyOptions, LocalSourceFilterOptions,
+    LocalSourceSaveOptions, ParentOptions, PathList, SnapshotOptions, repofile::SnapshotFile,
 };
 
 /// `backup` subcommand
@@ -73,11 +73,6 @@ pub struct BackupCmd {
     #[merge(strategy=conflate::option::overwrite_none)]
     as_path: Option<PathBuf>,
 
-    /// Ignore save options
-    #[clap(flatten)]
-    #[serde(flatten)]
-    ignore_save_opts: LocalSourceSaveOptions,
-
     /// Don't scan the backup source for its size - this disables ETA estimation for backup.
     #[clap(long)]
     #[merge(strategy=conflate::bool::overwrite_false)]
@@ -93,16 +88,16 @@ pub struct BackupCmd {
     #[merge(strategy=conflate::bool::overwrite_false)]
     long: bool,
 
-    /// Don't show any output
-    #[clap(long, conflicts_with_all = ["json", "long"])]
-    #[merge(strategy=conflate::bool::overwrite_false)]
-    quiet: bool,
-
     /// Initialize repository, if it doesn't exist yet
     #[clap(long)]
     #[merge(strategy=conflate::bool::overwrite_false)]
     init: bool,
 
+    /// Node save options
+    #[clap(flatten, next_help_heading = "Node modification options")]
+    #[serde(flatten)]
+    ignore_save_opts: LocalSourceSaveOptions,
+
     /// Parent processing options
     #[clap(flatten, next_help_heading = "Options for parent processing")]
     #[serde(flatten)]
@@ -111,6 +106,11 @@ pub struct BackupCmd {
     /// Exclude options
     #[clap(flatten, next_help_heading = "Exclude options")]
     #[serde(flatten)]
+    excludes: Excludes,
+
+    /// Exclude options for local source
+    #[clap(flatten, next_help_heading = "Exclude options for local source")]
+    #[serde(flatten)]
     ignore_filter_opts: LocalSourceFilterOptions,
 
     /// Snapshot options
@@ -211,7 +211,7 @@ impl Runnable for BackupCmd {
 }
 
 impl BackupCmd {
-    fn inner_run(&self, repo: CliRepo) -> Result<()> {
+    fn inner_run(&self, repo: Repo) -> Result<()> {
         let config = RUSTIC_APP.config();
 
         // Initialize repository if --init is set and it is not yet initialized
@@ -222,9 +222,14 @@ impl BackupCmd {
                     repo.name
                 );
             }
-            init(repo.0, &self.key_opts, &self.config_opts)?
+            init(
+                repo,
+                &config.repository.credential_opts,
+                &self.key_opts,
+                &self.config_opts,
+            )?
         } else {
-            repo.open()?
+            repo.open(&config.repository.credential_opts)?
         }
         .to_indexed_ids()?;
 
@@ -318,11 +323,7 @@ impl BackupCmd {
         hooks.with_env(&hooks_variables)
     }
 
-    fn backup_snapshot<P: ProgressBars, S: IndexedIds>(
-        mut self,
-        source: PathList,
-        repo: &Repository<P, S>,
-    ) -> Result<()> {
+    fn backup_snapshot(mut self, source: PathList, repo: &IndexedIdsRepo) -> Result<()> {
         let config = RUSTIC_APP.config();
         let snapshot_opts = &config.backup.snapshots;
         if let Some(path) = &self.as_path {
@@ -331,14 +332,13 @@ impl BackupCmd {
                 bail!("as-path only works with a single source!");
             }
             // merge Options from config file using as_path, if given
-            if let Some(path) = path.as_os_str().to_str() {
-                if let Some(idx) = snapshot_opts
+            if let Some(path) = path.as_os_str().to_str()
+                && let Some(idx) = snapshot_opts
                     .iter()
                     .position(|opt| opt.sources == vec![path])
-                {
-                    info!("merging snapshot=\"{path}\" section from config file");
-                    self.merge(snapshot_opts[idx].clone());
-                }
+            {
+                info!("merging snapshot=\"{path}\" section from config file");
+                self.merge(snapshot_opts[idx].clone());
             }
         }
 
@@ -360,6 +360,7 @@ impl BackupCmd {
             .as_path(self.as_path)
             .parent_opts(parent_opts)
             .ignore_save_opts(self.ignore_save_opts)
+            .excludes(self.excludes)
             .ignore_filter_opts(self.ignore_filter_opts)
             .no_scan(self.no_scan)
             .dry_run(config.global.dry_run);
@@ -385,7 +386,7 @@ impl BackupCmd {
             fill_table(&snap, add_entry);
 
             println!("{table}");
-        } else if !self.quiet {
+        } else {
             let summary = snap.summary.as_ref().unwrap();
             info!(
                 "Files:       {} new, {} changed, {} unchanged",
@@ -450,7 +451,7 @@ fn publish_metrics(
         Metric {
             name: "rustic_backup_time",
             description: "Timestamp of this snapshot",
-            value: Float(snap.time.timestamp_millis() as f64 / 1000.),
+            value: Float(snap.time.timestamp().as_millisecond() as f64 / 1000.),
         },
         Metric {
             name: "rustic_backup_files_new",
@@ -545,12 +546,12 @@ fn publish_metrics(
         Metric {
             name: "rustic_backup_backup_start",
             description: "Start time of the backup. This may differ from the snapshot `time`.",
-            value: Float(summary.backup_start.timestamp_millis() as f64 / 1000.),
+            value: Float(summary.backup_start.timestamp().as_millisecond() as f64 / 1000.),
         },
         Metric {
             name: "rustic_backup_backup_end",
             description: "The time that the backup has been finished.",
-            value: Float(summary.backup_end.timestamp_millis() as f64 / 1000.),
+            value: Float(summary.backup_end.timestamp().as_millisecond() as f64 / 1000.),
         },
         Metric {
             name: "rustic_backup_backup_duration",
