@@ -2,9 +2,9 @@
 
 use crate::{
     Application, RUSTIC_APP, RusticConfig,
-    commands::init::init_password,
+    commands::init::init_credentials,
     helpers::table_with_titles,
-    repository::{CliIndexedRepo, CliRepo},
+    repository::{AllRepositoryOptions, IndexedRepo, Repo, get_snapots_from_ids},
     status_err,
 };
 use abscissa_core::{Command, FrameworkError, Runnable, Shutdown, config::Override};
@@ -18,22 +18,30 @@ use rustic_core::{CopySnapshot, Id, KeyOptions, repofile::SnapshotFile};
 /// `copy` subcommand
 #[derive(clap::Parser, Command, Default, Clone, Debug, Serialize, Deserialize, Merge)]
 pub struct CopyCmd {
-    /// Snapshots to copy. If none is given, use filter options to filter from all snapshots.
+    /// Snapshots to copy. If none is given, use filter options to filter from all snapshots
+    ///
+    /// Snapshots can be identified the following ways: "01a2b3c4" or "latest" or "latest~N" (N >= 0)
     #[clap(value_name = "ID")]
     #[serde(skip)]
     #[merge(skip)]
     ids: Vec<String>,
 
+    /// Target repository (can be specified multiple times)
+    #[clap(long = "target", value_name = "TARGET")]
+    #[merge(strategy=conflate::vec::overwrite_empty)]
+    targets: Vec<String>,
+
     /// Initialize non-existing target repositories
     #[clap(long)]
     #[serde(skip)]
     #[merge(skip)]
     init: bool,
 
-    /// Target repository (can be specified multiple times)
-    #[clap(long = "target", value_name = "TARGET")]
-    #[merge(strategy=conflate::vec::overwrite_empty)]
-    targets: Vec<String>,
+    /// Copy snapshots even if the target already contains the original snapshot
+    #[clap(long)]
+    #[serde(skip)]
+    #[merge(skip)]
+    force: bool,
 
     /// Key options (when using --init)
     #[clap(flatten, next_help_heading = "Key options (when using --init)")]
@@ -72,13 +80,10 @@ impl Runnable for CopyCmd {
 }
 
 impl CopyCmd {
-    fn inner_run(&self, repo: CliIndexedRepo) -> Result<()> {
+    fn inner_run(&self, repo: IndexedRepo) -> Result<()> {
         let config = RUSTIC_APP.config();
-        let mut snapshots = if self.ids.is_empty() {
-            repo.get_matching_snapshots(|sn| config.snapshot_filter.matches(sn))?
-        } else {
-            repo.get_snapshots(&self.ids)?
-        };
+        let config = config;
+        let mut snapshots = get_snapots_from_ids(&repo, &self.ids)?;
         // sort for nicer output
         snapshots.sort_unstable();
 
@@ -92,7 +97,7 @@ impl CopyCmd {
             }
             let target_opt = &target_config.repository;
             if let Err(err) =
-                target_opt.run(|target_repo| self.copy(&repo, target_repo, &snapshots))
+                target_opt.run(|target_repo| self.copy(&repo, target_repo, target_opt, &snapshots))
             {
                 error!("error copying to target: {err}");
             }
@@ -102,8 +107,9 @@ impl CopyCmd {
 
     fn copy(
         &self,
-        repo: &CliIndexedRepo,
-        target_repo: CliRepo,
+        repo: &IndexedRepo,
+        target_repo: Repo,
+        target_opt: &AllRepositoryOptions,
         snapshots: &[SnapshotFile],
     ) -> Result<()> {
         let config = RUSTIC_APP.config();
@@ -112,12 +118,12 @@ impl CopyCmd {
         let target_repo = if self.init && target_repo.config_id()?.is_none() {
             let mut config_dest = repo.config().clone();
             config_dest.id = Id::random().into();
-            let pass = init_password(&target_repo)?;
+            let pass = init_credentials(&target_opt.credential_opts)?;
             target_repo
                 .0
                 .init_with_config(&pass, &self.key_opts, config_dest)?
         } else {
-            target_repo.open()?
+            target_repo.open(&target_opt.credential_opts)?
         };
 
         if !repo.config().has_same_chunker(target_repo.config()) {
@@ -126,17 +132,25 @@ impl CopyCmd {
             );
         }
 
-        let snaps = target_repo.relevant_copy_snapshots(
-            |sn| !self.ids.is_empty() || config.snapshot_filter.matches(sn),
-            snapshots,
-        )?;
+        let snaps = if self.force {
+            snapshots
+                .iter()
+                .cloned()
+                .map(|sn| CopySnapshot { sn, relevant: true })
+                .collect()
+        } else {
+            target_repo.relevant_copy_snapshots(
+                |sn| !self.ids.is_empty() || config.snapshot_filter.matches(sn),
+                snapshots,
+            )?
+        };
 
         let mut table =
             table_with_titles(["ID", "Time", "Host", "Label", "Tags", "Paths", "Status"]);
         for CopySnapshot { relevant, sn } in &snaps {
             let tags = sn.tags.formatln();
             let paths = sn.paths.formatln();
-            let time = sn.time.format("%Y-%m-%d %H:%M:%S").to_string();
+            let time = config.global.format_time(&sn.time).to_string();
             _ = table.add_row([
                 &sn.id.to_string(),
                 &time,
