@@ -4,7 +4,7 @@ use std::path::{Path, PathBuf};
 
 use crate::{
     Application, RUSTIC_APP,
-    repository::{CliIndexedRepo, get_global_grouped_snapshots},
+    repository::{IndexedRepo, get_global_grouped_snapshots},
     status_err,
 };
 
@@ -37,6 +37,8 @@ pub(crate) struct FindCmd {
     path: Option<PathBuf>,
 
     /// Snapshots to search in. If none is given, use filter options to filter from all snapshots
+    ///
+    /// Snapshots can be identified the following ways: "01a2b3c4" or "latest" or "latest~N" (N >= 0)
     #[clap(value_name = "ID")]
     ids: Vec<String>,
 
@@ -67,21 +69,19 @@ impl Runnable for FindCmd {
 }
 
 impl FindCmd {
-    fn inner_run(&self, repo: CliIndexedRepo) -> Result<()> {
-        let groups = get_global_grouped_snapshots(&repo, &self.ids)?;
-        for (group, mut snapshots) in groups {
-            snapshots.sort_unstable();
-            if !group.is_empty() {
-                println!("\nsearching in snapshots group {group}...");
+    fn inner_run(&self, repo: IndexedRepo) -> Result<()> {
+        let grouped = get_global_grouped_snapshots(&repo, &self.ids)?;
+        for group in grouped.groups {
+            let mut snaps = group.items;
+            let key = group.group_key;
+            snaps.sort_unstable();
+            if !key.is_empty() {
+                println!("\nsearching in snapshots group {key}...");
             }
-            let ids = snapshots.iter().map(|sn| sn.tree);
+            let ids = snaps.iter().map(|sn| sn.tree);
             if let Some(path) = &self.path {
                 let FindNode { nodes, matches } = repo.find_nodes_from_path(ids, path)?;
-                for (idx, g) in &matches
-                    .iter()
-                    .zip(snapshots.iter())
-                    .chunk_by(|(idx, _)| *idx)
-                {
+                for (idx, g) in &matches.iter().zip(snaps.iter()).chunk_by(|(idx, _)| *idx) {
                     self.print_identical_snapshots(idx.iter(), g.into_iter().map(|(_, sn)| sn));
                     if let Some(idx) = idx {
                         print_node(&nodes[*idx], path, self.numeric_id);
@@ -104,11 +104,7 @@ impl FindCmd {
                     nodes,
                     matches,
                 } = repo.find_matching_nodes(ids, &matches)?;
-                for (idx, g) in &matches
-                    .iter()
-                    .zip(snapshots.iter())
-                    .chunk_by(|(idx, _)| *idx)
-                {
+                for (idx, g) in &matches.iter().zip(snaps.iter()).chunk_by(|(idx, _)| *idx) {
                     self.print_identical_snapshots(idx.iter(), g.into_iter().map(|(_, sn)| sn));
                     for (path_idx, node_idx) in idx {
                         print_node(&nodes[*node_idx], &paths[*path_idx], self.numeric_id);
@@ -124,18 +120,19 @@ impl FindCmd {
         mut idx: impl Iterator,
         mut g: impl Iterator<Item = &'a SnapshotFile>,
     ) {
+        let config = RUSTIC_APP.config();
         let empty_result = idx.next().is_none();
         let not = if empty_result { "not " } else { "" };
         if self.show_misses || !empty_result {
             if self.all {
                 for sn in g {
-                    let time = sn.time.format("%Y-%m-%d %H:%M:%S");
+                    let time = config.global.format_time(&sn.time);
                     println!("{not}found in {} from {time}", sn.id);
                 }
             } else {
                 let sn = g.next().unwrap();
                 let count = g.count();
-                let time = sn.time.format("%Y-%m-%d %H:%M:%S");
+                let time = config.global.format_time(&sn.time);
                 match count {
                     0 => println!("{not}found in {} from {time}", sn.id),
                     count => println!("{not}found in {} from {time} (+{count})", sn.id),
