@@ -2,15 +2,14 @@
 
 use crate::{
     Application, RUSTIC_APP,
-    repository::{CliOpenRepo, get_filtered_snapshots},
+    repository::{OpenRepo, get_snapots_from_ids},
     status_err,
 };
 use abscissa_core::{Command, Runnable, Shutdown};
 use anyhow::Result;
+use jiff::Zoned;
 use log::info;
 
-use chrono::Local;
-
 use rustic_core::{SnapshotOptions, last_modified_node, repofile::SnapshotFile};
 
 /// `merge` subcommand
@@ -47,15 +46,11 @@ impl Runnable for MergeCmd {
 }
 
 impl MergeCmd {
-    fn inner_run(&self, repo: CliOpenRepo) -> Result<()> {
+    fn inner_run(&self, repo: OpenRepo) -> Result<()> {
         let config = RUSTIC_APP.config();
         let repo = repo.to_indexed_ids()?;
 
-        let snapshots = if self.ids.is_empty() {
-            get_filtered_snapshots(&repo)?
-        } else {
-            repo.get_snapshots(&self.ids)?
-        };
+        let snapshots = get_snapots_from_ids(&repo, &self.ids)?;
 
         // Handle dry-run mode
         if config.global.dry_run {
@@ -73,11 +68,11 @@ impl MergeCmd {
         info!("saved new snapshot as {}.", snap.id);
 
         if self.delete {
-            let now = Local::now();
+            let now = Zoned::now();
             // TODO: Maybe use this check in repo.delete_snapshots?
             let snap_ids: Vec<_> = snapshots
                 .iter()
-                .filter(|sn| !sn.must_keep(now))
+                .filter(|sn| !sn.must_keep(&now))
                 .map(|sn| sn.id)
                 .collect();
             repo.delete_snapshots(&snap_ids)?;
