@@ -1,9 +1,9 @@
-//! `smapshot` subcommand
+//! `snapshot` subcommand
 
 use crate::{
     Application, RUSTIC_APP,
     helpers::{bold_cell, bytes_size_to_string, table, table_right_from},
-    repository::{CliOpenRepo, get_global_grouped_snapshots},
+    repository::{OpenRepo, get_global_grouped_snapshots},
     status_err,
 };
 
@@ -11,14 +11,14 @@ use abscissa_core::{Command, Runnable, Shutdown};
 use anyhow::Result;
 use comfy_table::Cell;
 use derive_more::From;
-use humantime::format_duration;
 use itertools::Itertools;
-use serde::Serialize;
+use jiff::SignedDuration;
 
 use rustic_core::{
-    Progress, ProgressBars, SnapshotGroup,
+    Group, ProgressBars, ProgressType, SnapshotGroup,
     repofile::{DeleteOption, SnapshotFile},
 };
+use serde::Serialize;
 
 #[cfg(feature = "tui")]
 use crate::commands::tui;
@@ -27,6 +27,8 @@ use crate::commands::tui;
 #[derive(clap::Parser, Command, Debug)]
 pub(crate) struct SnapshotCmd {
     /// Snapshots to show. If none is given, use filter options to filter from all snapshots
+    ///
+    /// Snapshots can be identified the following ways: "01a2b3c4" or "latest" or "latest~N" (N >= 0)
     #[clap(value_name = "ID")]
     ids: Vec<String>,
 
@@ -62,7 +64,7 @@ impl Runnable for SnapshotCmd {
 }
 
 impl SnapshotCmd {
-    fn inner_run(&self, repo: CliOpenRepo) -> Result<()> {
+    fn inner_run(&self, repo: OpenRepo) -> Result<()> {
         #[cfg(feature = "tui")]
         if self.interactive {
             return tui::run(|progress| {
@@ -70,7 +72,10 @@ impl SnapshotCmd {
                 config
                     .repository
                     .run_indexed_with_progress(progress.clone(), |repo| {
-                        let p = progress.progress_spinner("starting rustic in interactive mode...");
+                        let p = progress.progress(
+                            ProgressType::Spinner,
+                            "starting rustic in interactive mode...",
+                        );
                         p.finish();
                         // create app and run it
                         let snapshots = tui::Snapshots::new(
@@ -83,69 +88,35 @@ impl SnapshotCmd {
             });
         }
 
-        let groups = get_global_grouped_snapshots(&repo, &self.ids)?;
+        let groups = get_global_grouped_snapshots(&repo, &self.ids)?.groups;
 
         if self.json {
-            #[derive(Serialize, From)]
-            struct SnapshotsGroup {
-                group_key: SnapshotGroup,
-                snapshots: Vec<SnapshotFile>,
-            }
-            let groups: Vec<SnapshotsGroup> = groups.into_iter().map(|g| g.into()).collect();
             let mut stdout = std::io::stdout();
             if groups.len() == 1 && groups[0].group_key.is_empty() {
                 // we don't use grouping, only output snapshots list
-                serde_json::to_writer_pretty(&mut stdout, &groups[0].snapshots)?;
+                serde_json::to_writer_pretty(&mut stdout, &groups[0].items)?;
             } else {
+                #[derive(Serialize, From)]
+                struct SnapshotsGroup {
+                    group_key: SnapshotGroup,
+                    snapshots: Vec<SnapshotFile>,
+                }
+                let groups: Vec<SnapshotsGroup> = groups
+                    .into_iter()
+                    .map(|g| (g.group_key, g.items).into())
+                    .collect();
                 serde_json::to_writer_pretty(&mut stdout, &groups)?;
             }
             return Ok(());
         }
 
         let mut total_count = 0;
-        for (group_key, snapshots) in groups {
+        for Group { group_key, items } in groups {
             if !group_key.is_empty() {
                 println!("\nsnapshots for {group_key}");
             }
-            let count = snapshots.len();
-
-            if self.long {
-                for snap in snapshots {
-                    let mut table = table();
-
-                    let add_entry = |title: &str, value: String| {
-                        _ = table.add_row([bold_cell(title), Cell::new(value)]);
-                    };
-                    fill_table(&snap, add_entry);
-
-                    println!("{table}");
-                    println!();
-                }
-            } else {
-                let mut table = table_right_from(
-                    6,
-                    [
-                        "ID", "Time", "Host", "Label", "Tags", "Paths", "Files", "Dirs", "Size",
-                    ],
-                );
-
-                if self.all {
-                    // Add all snapshots to output table
-                    _ = table.add_rows(snapshots.into_iter().map(|sn| snap_to_table(&sn, 0)));
-                } else {
-                    // Group snapshts by treeid and output into table
-                    _ = table.add_rows(
-                        snapshots
-                            .into_iter()
-                            .chunk_by(|sn| sn.tree)
-                            .into_iter()
-                            .map(|(_, mut g)| snap_to_table(&g.next().unwrap(), g.count())),
-                    );
-                }
-                println!("{table}");
-            }
-            println!("{count} snapshot(s)");
-            total_count += count;
+            total_count += items.len();
+            print_snapshots(items, self.long, self.all);
         }
         println!();
         println!("total: {total_count} snapshot(s)");
@@ -154,10 +125,51 @@ impl SnapshotCmd {
     }
 }
 
+pub fn print_snapshots(snapshots: Vec<SnapshotFile>, long: bool, all: bool) {
+    let count = snapshots.len();
+    if long {
+        for snap in snapshots {
+            let mut table = table();
+
+            let add_entry = |title: &str, value: String| {
+                _ = table.add_row([bold_cell(title), Cell::new(value)]);
+            };
+            fill_table(&snap, add_entry);
+
+            println!("{table}");
+            println!();
+        }
+    } else {
+        let mut table = table_right_from(
+            6,
+            [
+                "ID", "Time", "Host", "Label", "Tags", "Paths", "Files", "Dirs", "Size",
+            ],
+        );
+
+        if all {
+            // Add all snapshots to output table
+            _ = table.add_rows(snapshots.into_iter().map(|sn| snap_to_table(&sn, 0)));
+        } else {
+            // Group snapshts by treeid and output into table
+            _ = table.add_rows(
+                snapshots
+                    .into_iter()
+                    .chunk_by(|sn| sn.tree)
+                    .into_iter()
+                    .map(|(_, mut g)| snap_to_table(&g.next().unwrap(), g.count())),
+            );
+        }
+        println!("{table}");
+    }
+    println!("{count} snapshot(s)");
+}
+
 pub fn snap_to_table(sn: &SnapshotFile, count: usize) -> [String; 9] {
+    let config = RUSTIC_APP.config();
     let tags = sn.tags.formatln();
     let paths = sn.paths.formatln();
-    let time = sn.time.format("%Y-%m-%d %H:%M:%S");
+    let time = config.global.format_time(&sn.time);
     let (files, dirs, size) = sn.summary.as_ref().map_or_else(
         || ("?".to_string(), "?".to_string(), "?".to_string()),
         |s| {
@@ -186,22 +198,23 @@ pub fn snap_to_table(sn: &SnapshotFile, count: usize) -> [String; 9] {
 }
 
 pub fn fill_table(snap: &SnapshotFile, mut add_entry: impl FnMut(&str, String)) {
+    let config = RUSTIC_APP.config();
     add_entry("Snapshot", snap.id.to_hex().to_string());
     // note that if original was not set, it is set to snap.id by the load process
-    if let Some(original) = snap.original {
-        if original != snap.id {
-            add_entry("Original ID", original.to_hex().to_string());
-        }
+    if let Some(original) = snap.original
+        && original != snap.id
+    {
+        add_entry("Original ID", original.to_hex().to_string());
     }
-    add_entry("Time", snap.time.format("%Y-%m-%d %H:%M:%S").to_string());
+    add_entry("Time", config.global.format_time(&snap.time).to_string());
     add_entry("Generated by", snap.program_version.clone());
     add_entry("Host", snap.hostname.clone());
     add_entry("Label", snap.label.clone());
     add_entry("Tags", snap.tags.formatln());
-    let delete = match snap.delete {
+    let delete = match &snap.delete {
         DeleteOption::NotSet => "not set".to_string(),
         DeleteOption::Never => "never".to_string(),
-        DeleteOption::After(t) => format!("after {}", t.format("%Y-%m-%d %H:%M:%S")),
+        DeleteOption::After(t) => format!("after {}", config.global.format_time(t)),
     };
     add_entry("Delete", delete);
     add_entry("Paths", snap.paths.formatln());
@@ -253,12 +266,12 @@ pub fn fill_table(snap: &SnapshotFile, mut add_entry: impl FnMut(&str, String))
         add_entry("Added to repo", written);
 
         let duration = format!(
-            "backup start: {} / backup end: {} / backup duration: {}\n\
-            total duration: {}",
-            summary.backup_start.format("%Y-%m-%d %H:%M:%S"),
-            summary.backup_end.format("%Y-%m-%d %H:%M:%S"),
-            format_duration(std::time::Duration::from_secs_f64(summary.backup_duration)),
-            format_duration(std::time::Duration::from_secs_f64(summary.total_duration))
+            "backup start: {} / backup end: {} / backup duration: {:#}\n\
+            total duration: {:#}",
+            config.global.format_time(&summary.backup_start),
+            config.global.format_time(&summary.backup_end),
+            SignedDuration::from_secs_f64(summary.backup_duration),
+            SignedDuration::from_secs_f64(summary.total_duration),
         );
         add_entry("Duration", duration);
     }
