@@ -1,14 +1,15 @@
 //! `tag` subcommand
 use abscissa_core::{Command, Runnable};
-use rustic_core::StringList;
+use rustic_core::{StringList, repofile::SnapshotModification};
 
 use crate::commands::rewrite::RewriteCmd;
 
 /// `tag` subcommand
 #[derive(clap::Parser, Command, Debug)]
 pub(crate) struct TagCmd {
-    /// Snapshots to change tags. If none is given, use filter to filter from all
-    /// snapshots.
+    /// Snapshots to change tags. If none is given, use filter to filter from all snapshots
+    ///
+    /// Snapshots can be identified the following ways: "01a2b3c4" or "latest" or "latest~N" (N >= 0)
     #[clap(value_name = "ID")]
     ids: Vec<String>,
 
@@ -37,11 +38,14 @@ pub(crate) struct TagCmd {
 
 impl Runnable for TagCmd {
     fn run(&self) {
+        let modification = SnapshotModification::default()
+            .add_tags(self.add.clone())
+            .remove_tags(self.remove.clone())
+            .set_tags(self.set.clone());
         let rewrite = RewriteCmd {
             ids: self.ids.clone(),
-            add_tags: self.add.clone(),
-            remove_tags: self.remove.clone(),
-            set_tags: self.set.clone(),
+            modification,
+            forget: true,
             ..Default::default()
         };
         rewrite.run();
