@@ -7,7 +7,7 @@ use std::net::ToSocketAddrs;
 
 use crate::{
     Application, RUSTIC_APP, RusticConfig,
-    repository::{CliIndexedRepo, get_filtered_snapshots},
+    repository::{IndexedRepo, get_filtered_snapshots},
     status_err,
 };
 use abscissa_core::{Command, FrameworkError, Runnable, Shutdown, config::Override};
@@ -34,7 +34,7 @@ pub struct WebDavCmd {
     #[merge(strategy=conflate::option::overwrite_none)]
     path_template: Option<String>,
 
-    /// The time template to use to display times in the path template. See https://docs.rs/chrono/latest/chrono/format/strftime/index.html for format options. [default: "%Y-%m-%d_%H-%M-%S"]
+    /// The time template to use to display times in the path template. See https://pubs.opengroup.org/onlinepubs/009695399/functions/strftime.html for format options. [default: "%Y-%m-%d_%H-%M-%S"]
     #[clap(long)]
     #[merge(strategy=conflate::option::overwrite_none)]
     time_template: Option<String>,
@@ -50,6 +50,8 @@ pub struct WebDavCmd {
     file_access: Option<String>,
 
     /// Specify directly which snapshot/path to serve
+    ///
+    /// Snapshot can be identified the following ways: "01a2b3c4" or "latest" or "latest~N" (N >= 0)
     #[clap(value_name = "SNAPSHOT[:PATH]")]
     #[merge(strategy=conflate::option::overwrite_none)]
     snapshot_path: Option<String>,
@@ -85,7 +87,7 @@ impl WebDavCmd {
     /// be careful about self VS RUSTIC_APP.config() usage
     /// only the RUSTIC_APP.config() involves the TOML and ENV merged configurations
     /// see https://github.com/rustic-rs/rustic/issues/1242
-    fn inner_run(&self, repo: CliIndexedRepo) -> Result<()> {
+    fn inner_run(&self, repo: IndexedRepo) -> Result<()> {
         let config = RUSTIC_APP.config();
 
         let path_template = config
