@@ -15,15 +15,12 @@ use conflate::Merge;
 use dialoguer::Password;
 use rustic_backend::BackendOptions;
 use rustic_core::{
-    FullIndex, IndexedStatus, Open, OpenStatus, ProgressBars, Repository, RepositoryOptions,
-    SnapshotGroup, SnapshotGroupCriterion, repofile::SnapshotFile,
+    CredentialOptions, Credentials, Grouped, IndexedFullStatus, IndexedIdsStatus, Open, OpenStatus,
+    ProgressBars, Repository, RepositoryOptions, SnapshotGroupCriterion, repofile::SnapshotFile,
 };
 use serde::{Deserialize, Serialize};
 
-use crate::{
-    RUSTIC_APP,
-    config::{hooks::Hooks, progress_options::ProgressOptions},
-};
+use crate::{RUSTIC_APP, config::hooks::Hooks};
 
 pub(super) mod constants {
     pub(super) const MAX_PASSWORD_RETRIES: usize = 5;
@@ -42,27 +39,27 @@ pub struct AllRepositoryOptions {
     #[serde(flatten)]
     pub repo: RepositoryOptions,
 
+    /// Credential options
+    #[clap(flatten, next_help_heading = "credential options")]
+    #[serde(flatten)]
+    pub credential_opts: CredentialOptions,
+
     /// Hooks
     #[clap(skip)]
     pub hooks: Hooks,
 }
 
-pub type CliRepo = RusticRepo<ProgressOptions>;
-pub type CliOpenRepo = Repository<ProgressOptions, OpenStatus>;
-pub type RusticIndexedRepo<P> = Repository<P, IndexedStatus<FullIndex, OpenStatus>>;
-pub type CliIndexedRepo = RusticIndexedRepo<ProgressOptions>;
-
 impl AllRepositoryOptions {
-    pub fn repository<P>(&self, po: P) -> Result<RusticRepo<P>> {
+    pub fn repository(&self, po: impl ProgressBars) -> Result<Repo> {
         let backends = self.be.to_backends()?;
         let repo = Repository::new_with_progress(&self.repo, &backends, po)?;
-        Ok(RusticRepo(repo))
+        Ok(Repo(repo))
     }
 
-    pub fn run_with_progress<P: Clone + ProgressBars, T>(
+    pub fn run_with_progress<T>(
         &self,
-        po: P,
-        f: impl FnOnce(RusticRepo<P>) -> Result<T>,
+        po: impl ProgressBars,
+        f: impl FnOnce(Repo) -> Result<T>,
     ) -> Result<T> {
         let hooks = self
             .hooks
@@ -74,76 +71,85 @@ impl AllRepositoryOptions {
         hooks.use_with(|| f(self.repository(po)?))
     }
 
-    pub fn run<T>(&self, f: impl FnOnce(CliRepo) -> Result<T>) -> Result<T> {
+    pub fn run<T>(&self, f: impl FnOnce(Repo) -> Result<T>) -> Result<T> {
         let po = RUSTIC_APP.config().global.progress_options;
         self.run_with_progress(po, f)
     }
 
-    pub fn run_open<T>(&self, f: impl FnOnce(CliOpenRepo) -> Result<T>) -> Result<T> {
-        self.run(|repo| f(repo.open()?))
+    pub fn run_open<T>(&self, f: impl FnOnce(OpenRepo) -> Result<T>) -> Result<T> {
+        self.run(|repo| f(repo.open(&self.credential_opts)?))
     }
 
     pub fn run_open_or_init_with<T: Clone>(
         &self,
         do_init: bool,
-        init: impl FnOnce(CliRepo) -> Result<CliOpenRepo>,
-        f: impl FnOnce(CliOpenRepo) -> Result<T>,
+        init: impl FnOnce(Repo) -> Result<OpenRepo>,
+        f: impl FnOnce(OpenRepo) -> Result<T>,
     ) -> Result<T> {
-        self.run(|repo| f(repo.open_or_init_repository_with(do_init, init)?))
+        self.run(|repo| {
+            f(repo.open_or_init_repository_with(&self.credential_opts, do_init, init)?)
+        })
     }
 
-    pub fn run_indexed_with_progress<P: Clone + ProgressBars, T>(
+    pub fn run_indexed_with_progress<T>(
         &self,
-        po: P,
-        f: impl FnOnce(RusticIndexedRepo<P>) -> Result<T>,
+        po: impl ProgressBars,
+        f: impl FnOnce(IndexedRepo) -> Result<T>,
     ) -> Result<T> {
-        self.run_with_progress(po, |repo| f(repo.indexed()?))
+        self.run_with_progress(po, |repo| f(repo.indexed(&self.credential_opts)?))
     }
 
-    pub fn run_indexed<T>(&self, f: impl FnOnce(CliIndexedRepo) -> Result<T>) -> Result<T> {
-        self.run(|repo| f(repo.indexed()?))
+    pub fn run_indexed<T>(&self, f: impl FnOnce(IndexedRepo) -> Result<T>) -> Result<T> {
+        self.run(|repo| f(repo.indexed(&self.credential_opts)?))
     }
 }
 
-#[derive(Debug)]
-pub struct RusticRepo<P>(pub Repository<P, ()>);
+pub type OpenRepo = Repository<OpenStatus>;
+pub type IndexedRepo = Repository<IndexedFullStatus>;
+pub type IndexedIdsRepo = Repository<IndexedIdsStatus>;
 
-impl<P> Deref for RusticRepo<P> {
-    type Target = Repository<P, ()>;
+#[derive(Debug)]
+pub struct Repo(pub Repository<()>);
+
+impl Deref for Repo {
+    type Target = Repository<()>;
     fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
 
-impl<P: Clone + ProgressBars> RusticRepo<P> {
-    pub fn open(self) -> Result<Repository<P, OpenStatus>> {
-        match self.0.password()? {
-            // if password is given, directly return the result of find_key_in_backend and don't retry
-            Some(pass) => {
-                return Ok(self.0.open_with_password(&pass)?);
-            }
+impl Repo {
+    pub fn open(self, credential_opts: &CredentialOptions) -> Result<OpenRepo> {
+        match credential_opts.credentials()? {
+            // if credentials are given, directly open the repository and don't retry
+            Some(credentials) => Ok(self.0.open(&credentials)?),
             None => {
                 for _ in 0..constants::MAX_PASSWORD_RETRIES {
                     let pass = Password::new()
                         .with_prompt("enter repository password")
                         .allow_empty_password(true)
                         .interact()?;
-                    match self.0.clone().open_with_password(&pass) {
+                    match self
+                        .0
+                        .clone() // needed; else we move repo in a loop
+                        .open(&Credentials::Password(pass))
+                    {
                         Ok(repo) => return Ok(repo),
                         Err(err) if err.is_incorrect_password() => continue,
                         Err(err) => return Err(err.into()),
                     }
                 }
+                Err(anyhow!("incorrect password"))
             }
         }
-        Err(anyhow!("incorrect password"))
     }
 
     fn open_or_init_repository_with(
         self,
+        credential_opts: &CredentialOptions,
         do_init: bool,
-        init: impl FnOnce(Self) -> Result<Repository<P, OpenStatus>>,
-    ) -> Result<Repository<P, OpenStatus>> {
+        init: impl FnOnce(Self) -> Result<OpenRepo>,
+    ) -> Result<OpenRepo> {
         let dry_run = RUSTIC_APP.config().global.check_index;
         // Initialize repository if --init is set and it is not yet initialized
         let repo = if do_init && self.0.config_id()?.is_none() {
@@ -155,13 +161,13 @@ impl<P: Clone + ProgressBars> RusticRepo<P> {
             }
             init(self)?
         } else {
-            self.open()?
+            self.open(credential_opts)?
         };
         Ok(repo)
     }
 
-    fn indexed(self) -> Result<Repository<P, IndexedStatus<FullIndex, OpenStatus>>> {
-        let open = self.open()?;
+    fn indexed(self, credential_opts: &CredentialOptions) -> Result<IndexedRepo> {
+        let open = self.open(credential_opts)?;
         let check_index = RUSTIC_APP.config().global.check_index;
         let repo = if check_index {
             open.to_indexed_checked()
@@ -172,34 +178,51 @@ impl<P: Clone + ProgressBars> RusticRepo<P> {
     }
 }
 
-pub fn get_filtered_snapshots<P: ProgressBars, S: Open>(
-    repo: &Repository<P, S>,
+// get snapshots from ids allowing `latest`, if empty use all snapshots respecting the filters.
+pub fn get_snapots_from_ids<S: Open>(
+    repo: &Repository<S>,
+    ids: &[String],
 ) -> Result<Vec<SnapshotFile>> {
+    let config = RUSTIC_APP.config();
+    let snapshots = if ids.is_empty() {
+        get_filtered_snapshots(repo)?
+    } else {
+        repo.get_snapshots_from_strs(ids, |sn| config.snapshot_filter.matches(sn))?
+    };
+    Ok(snapshots)
+}
+
+// get all snapshots respecting the filters
+pub fn get_filtered_snapshots<S: Open>(repo: &Repository<S>) -> Result<Vec<SnapshotFile>> {
     let config = RUSTIC_APP.config();
     let mut snapshots = repo.get_matching_snapshots(|sn| config.snapshot_filter.matches(sn))?;
     config.snapshot_filter.post_process(&mut snapshots);
     Ok(snapshots)
 }
 
-pub fn get_global_grouped_snapshots<P: ProgressBars, S: Open>(
-    repo: &Repository<P, S>,
+pub fn get_global_grouped_snapshots<S: Open>(
+    repo: &Repository<S>,
     ids: &[String],
-) -> Result<Vec<(SnapshotGroup, Vec<SnapshotFile>)>> {
+) -> Result<Grouped<SnapshotFile>> {
     let config = RUSTIC_APP.config();
     get_grouped_snapshots(repo, config.global.group_by.unwrap_or_default(), ids)
 }
 
-pub fn get_grouped_snapshots<P: ProgressBars, S: Open>(
-    repo: &Repository<P, S>,
+pub fn get_grouped_snapshots<S: Open>(
+    repo: &Repository<S>,
     group_by: SnapshotGroupCriterion,
     ids: &[String],
-) -> Result<Vec<(SnapshotGroup, Vec<SnapshotFile>)>> {
+) -> Result<Grouped<SnapshotFile>> {
     let config = RUSTIC_APP.config();
-    let mut groups =
-        repo.get_snapshot_group(ids, group_by, |sn| config.snapshot_filter.matches(sn))?;
-
-    for (_, snaps) in &mut groups {
-        config.snapshot_filter.post_process(snaps);
+    let snapshots = if ids.is_empty() {
+        repo.get_matching_snapshots(|sn| config.snapshot_filter.matches(sn))?
+    } else {
+        repo.get_snapshots_from_strs(ids, |sn| config.snapshot_filter.matches(sn))?
+    };
+    let mut group = Grouped::from_items(snapshots, group_by);
+    for group in &mut group.groups {
+        config.snapshot_filter.post_process(&mut group.items);
     }
-    Ok(groups)
+
+    Ok(group)
 }
